#if defined(SUNOS55) || defined(SUNOS56)

#include <sys/param.h>
#include <sys/stream.h>
#include <sys/dditypes.h>
#include <sys/ethernet.h>
#include <nlist.h>
#include <fcntl.h>
#include <stdio.h>
#include <kvm.h>

#ifdef SUNOS55
#include "if-solaris25.h"
#endif
#ifdef SUNOS56
#include "if-solaris26.h"
#endif

struct nlist nl[] = {
#define X_IE		0
	{ "iedev"	},
#define X_LE		1
	{ "ledev"	},
#define X_QE		2
	{ "qeup"	},
#define X_HME		3
	{ "hmeup"	},
#define X_XX		4
	{ 0		}
};

extern char	*hostName;
extern char	*programName;

extern int	verbose;

checkInterfaces()
{
	kvm_t *kd;

	if ((kd = kvm_open(NULL, NULL, NULL, O_RDONLY, programName)) == NULL)
		fatal("kvm_open", NULL);

	if (kvm_nlist(kd, nl) < 0)
		fatal("kvm_nlist", NULL);

	if (nl[X_IE].n_value != 0)
		checkIE(kd);

	if (nl[X_LE].n_value != 0)
		checkLE(kd);

	if (nl[X_QE].n_value != 0)
		checkQE(kd);

	if (nl[X_HME].n_value != 0)
		checkHME(kd);
	kvm_close(kd);
}

checkIE(kd)
kvm_t *kd;
{
	struct ie ie;
	struct dev_info di;
	u_long ieaddr, dipaddr;

	ieaddr = nl[X_IE].n_value;

	do {
		if (kvm_read(kd, ieaddr, (char *) &ie, sizeof(struct ie)) < 0)
			fatal("kvm_read: ie", NULL);

		dipaddr = (u_long) ie.ie_dip;
		ieaddr = (u_long) ie.ie_nextp;

		if (dipaddr == 0)
			continue;

		if (kvm_read(kd, dipaddr, (char *) &di, sizeof(struct dev_info)) < 0)
			continue;

		if (verbose) {
			printf("Interface ie%d: flags=0x%x\n",
			       di.devi_instance, ie.ie_flags);
		}

		if (ie.ie_flags & IEPROMISC) {
			printf("WARNING: %s INTERFACE ie%d IS IN PROMISCUOUS MODE.\n",
			       hostName, di.devi_instance);
		}
	} while (ieaddr != 0);
}

checkLE(kd)
kvm_t *kd;
{
	struct le le;
	struct dev_info di;
	u_long leaddr, dipaddr;

	leaddr = nl[X_LE].n_value;

	do {
		if (kvm_read(kd, leaddr, (char *) &le, sizeof(struct le)) < 0)
			fatal("kvm_read: le", NULL);

		dipaddr = (u_long) le.le_dip;
		leaddr = (u_long) le.le_nextp;

		if (dipaddr == 0)
			continue;

		if (kvm_read(kd, dipaddr, (char *) &di, sizeof(struct dev_info)) < 0)
			continue;

		if (verbose) {
			printf("Interface le%d: flags=0x%x\n",
			       di.devi_instance, le.le_flags);
		}

		if (le.le_flags & LEPROMISC) {
			printf("WARNING: %s INTERFACE le%d IS IN PROMISCUOUS MODE.\n",
			       hostName, di.devi_instance);
		}
	} while (leaddr != 0);
}

checkQE(kd)
kvm_t *kd;
{
	struct qe qe;
	struct dev_info di;
	u_long qeaddr, dipaddr;

	qeaddr = nl[X_QE].n_value;

	do {
		if (kvm_read(kd, qeaddr, (char *) &qe, sizeof(struct qe)) < 0)
			fatal("kvm_read: qe", NULL);

		dipaddr = (u_long) qe.qe_dip;
		qeaddr = (u_long) qe.qe_nextp;

		if (dipaddr == 0)
			continue;

		if (kvm_read(kd, dipaddr, (char *) &di, sizeof(struct dev_info)) < 0)
			continue;

		if (verbose) {
			printf("Interface qe%d: flags=0x%x\n",
			       di.devi_instance, qe.qe_flags);
		}

		if (qe.qe_flags & QEPROMISC) {
			printf("WARNING: %s INTERFACE qe%d IS IN PROMISCUOUS MODE.\n",
			       hostName, di.devi_instance);
		}
	} while (qeaddr != 0);
}
checkHME(kd)
kvm_t *kd;
{
	struct hme hme;
	struct dev_info di;
	u_long hmeaddr, dipaddr;
	hmeaddr = nl[X_HME].n_value;
	do {
		if (kvm_read(kd, hmeaddr, (char *) &hme, sizeof(struct hme)) < 0)
			fatal("kvm_read: hme", NULL);
		dipaddr = (u_long) hme.hme_dip;
		hmeaddr = (u_long) hme.hme_nextp;
		if (dipaddr == 0)
			continue;
		if (kvm_read(kd, dipaddr, (char *) &di, sizeof(struct dev_info)) < 0)
			continue;
		if (verbose) {
			printf("Interface hme%d: flags=0x%x\n",
			       di.devi_instance, hme.hme_flags);
		}
		if (hme.hme_flags & HMEPROMISC) {
			printf("WARNING: %s INTERFACE hme%d IS IN PROMISCUOUS MODE.\n",
			       hostName, di.devi_instance);
		}
	} while (hmeaddr != 0);
}

#endif /* SUNOS5 */
